Module: Actions

Defined in:
app/lib/actions/foreman_openbolt/poll_task_status.rb,
app/lib/actions/foreman_openbolt/cleanup_proxy_artifacts.rb

Overview

This is a Dynflow action that polls the status of an OpenBolt task job from the Smart Proxy. It periodically checks the job status until it is completed, then fetches the final results.

Defined Under Namespace

Modules: ForemanOpenbolt